@@ -55,7 +55,11 @@ XO = 0
 YO = 0
 
 # Pin settings.
-PIN_LENGTH = 200
+MIN_PIN_LENGTH = 200  # Minimum length of pins
+PIN_LENGTH_BASE = 0  # Whitespace length of pins
+PIN_LENGTH_PER_CHAR = 50  # Additional pin length per pin number character
+PIN_LENGTH_STYLE_MOD = 100  # Additional pin length for pin styles that encroach
+PIN_LENGTH_STYLES = {'CL', 'F', 'I', 'IC', 'L', 'V'}  # Pin styles that encroach
 PIN_SPACING = 100
 PIN_NUM_SIZE = 50  # Font size for pin numbers.
 PIN_NAME_SIZE = 50  # Font size for pin names.
@@ -282,7 +286,7 @@ def count_pin_slots(unit_pins):
     return num_slots
 
 
-def pins_bbox(unit_pins):
+def pins_bbox(unit_pins, pin_length):
     """Return the bounding box of a column of pins and their names."""
 
     if len(unit_pins) == 0:
@@ -293,7 +297,7 @@ def pins_bbox(unit_pins):
 
     # Compute bbox width adding the separation space before and after the pin name.
     # (The space after the pin name keeps pin names on opposing sides from colliding.)
-    width = width * PIN_NAME_SIZE + PIN_LENGTH + 2 * PIN_NAME_OFFSET
+    width = width * PIN_NAME_SIZE + pin_length + 2 * PIN_NAME_OFFSET
 
     # Make bounding box an integer number of pin spaces so pin connections are always on the grid.
     width = math.ceil(float(width) / PIN_SPACING) * PIN_SPACING
@@ -383,7 +387,7 @@ def balance_bboxes(bboxes):
             bboxes["bottom"][1][Y] = bal_bbox[1][Y]
 
 
-def calc_scrunch(bboxes):
+def calc_scrunch(bboxes, pin_length):
     """Return the "scrunch" for compressing the left/right sides under the top/bottom."""
 
     X, Y = 0, 1
@@ -393,13 +397,13 @@ def calc_scrunch(bboxes):
     lr_bbox = find_bbox_bbox(bboxes["left"], bboxes["right"])
     tb_bbox = find_bbox_bbox(bboxes["top"], bboxes["bottom"])
     tb_width = abs(tb_bbox[0][Y] - tb_bbox[1][Y])
-    lr_width = abs(lr_bbox[0][X] - lr_bbox[1][X]) - PIN_LENGTH
+    lr_width = abs(lr_bbox[0][X] - lr_bbox[1][X]) - pin_length
     scrunch = min(tb_width / 2, lr_width)
 
     return scrunch
 
 
-def draw_pins(unit_num, unit_pins, bbox, transform, side, push, fuzzy_match):
+def draw_pins(unit_num, unit_pins, bbox, transform, side, push, fuzzy_match, pin_length):
     """Draw a column of pins rotated/translated by the transform matrix."""
 
     # String to add pin definitions to.
@@ -409,7 +413,7 @@ def draw_pins(unit_num, unit_pins, bbox, transform, side, push, fuzzy_match):
     # bounding box (which should be at least as large). Half the difference
     # will be the offset needed to center the pins on the side of the symbol.
     Y = 1  # Index for Y coordinate.
-    pins_bb = pins_bbox(unit_pins)
+    pins_bb = pins_bbox(unit_pins, pin_length)
     height_offset = abs(bbox[0][Y] - bbox[1][Y]) - abs(pins_bb[0][Y] - pins_bb[1][Y])
     push = min(max(0.0, push), 1.0)
     if side in ("right", "top"):
@@ -457,7 +461,7 @@ def draw_pins(unit_num, unit_pins, bbox, transform, side, push, fuzzy_match):
                 num=pin_num,
                 x=int(draw_x),
                 y=int(draw_y),
-                length=PIN_LENGTH,
+                length=pin_length,
                 orientation=pin_side,
                 num_sz=num_size,
                 name_sz=PIN_NAME_SIZE,
@@ -542,93 +546,8 @@ def draw_symbol(
 ):
     """Add a symbol for a part to the library."""
 
-    # Start the part definition with the header.
-    part_defn = START_DEF.format(
-        name=part_num,
-        ref=part_ref_prefix,
-        pin_name_offset=PIN_NAME_OFFSET,
-        show_pin_number=SHOW_PIN_NUMBER and "Y" or "N",
-        show_pin_name=SHOW_PIN_NAME and "Y" or "N",
-        num_units=len(pin_data),
-    )
-
-    if center_symbol:
-        # Origin is in the center of the symbol.
-        text_justification = "C"
-        horiz_offset = 0
-        vert_offset = 50
-    else:
-        # Origin is on the top-most pin on the left side.
-        # Determine if there are pins across the top of the symbol.
-        # If so, right-justify the reference, part number, etc. so they don't
-        # run into the top pins. If not, stick with left-justification.
-        text_justification = "L"
-        horiz_offset = PIN_LENGTH
-        vert_offset = 250
-        for unit in list(pin_data.values()):
-            if "top" in list(unit.keys()):
-                text_justification = "R"
-                horiz_offset = PIN_LENGTH - 50
-                break
-
-    # Create the field that stores the part reference.
-    part_defn += REF_FIELD.format(
-        ref_prefix=part_ref_prefix or "U",
-        x=XO + horiz_offset,
-        y=YO + REF_Y_OFFSET + vert_offset,
-        text_justification=text_justification,
-        font_size=REF_SIZE,
-    )
-
-    # Create the field that stores the part number.
-    part_defn += PARTNUM_FIELD.format(
-        num=part_num or "",
-        x=XO + horiz_offset,
-        y=YO + PART_NUM_Y_OFFSET + vert_offset,
-        text_justification=text_justification,
-        font_size=PART_NUM_SIZE,
-    )
-
-    # Create the field that stores the part footprint.
-    part_defn += FOOTPRINT_FIELD.format(
-        footprint=part_footprint or "",
-        x=XO + horiz_offset,
-        y=YO + PART_FOOTPRINT_Y_OFFSET + vert_offset,
-        text_justification=text_justification,
-        font_size=PART_FOOTPRINT_SIZE,
-    )
-
-    # Create the field that stores the datasheet link.
-    part_defn += DATASHEET_FIELD.format(
-        datasheet=part_datasheet or "",
-        x=XO + horiz_offset,
-        y=YO + PART_DATASHEET_Y_OFFSET + vert_offset,
-        text_justification=text_justification,
-        font_size=PART_DATASHEET_SIZE,
-    )
-
-    # Create the field that stores the manufacturer part number.
-    if part_manf_num:
-        part_defn += MPN_FIELD.format(
-            manf_num=part_manf_num,
-            x=XO + horiz_offset,
-            y=YO + PART_MPN_Y_OFFSET + vert_offset,
-            text_justification=text_justification,
-            font_size=PART_MPN_SIZE,
-        )
-
-    # Create the field that stores the datasheet link.
-    if part_desc:
-        part_defn += DESC_FIELD.format(
-            desc=part_desc,
-            x=XO + horiz_offset,
-            y=YO + PART_DESC_Y_OFFSET + vert_offset,
-            text_justification=text_justification,
-            font_size=PART_DESC_SIZE,
-        )
-
-    # Start the section of the part definition that holds the part's units.
-    part_defn += START_DRAW
+    # Determine the maximum length of pin
+    pin_length = calculate_pin_length(pin_data, fuzzy_match)
 
     # Get a reference to the sort-key generation function for pins.
     pin_key_func = getattr(THIS_MODULE, "{}_key".format(sort_type))
@@ -636,23 +555,26 @@ def draw_symbol(
     # This is the sort-key generation function for unit names.
     unit_key_func = lambda x: zero_pad_nums(x[0])
 
-    # Now create the units that make up the part. Unit numbers go from 1
+    # The indices of the X and Y coordinates in a list of point coords.
+    X = 0
+    Y = 1
+
+    # Data for the units, used to draw everything
+    all_sides = ["left", "right", "top", "bottom"]
+    unitdata = {}
+
+    # Analyze the units that make up the part. Unit numbers go from 1
     # up to the number of units in the part. The units are sorted by their
     # names before assigning unit numbers.
     for unit_num, unit in enumerate(
         [p[1] for p in sorted(pin_data.items(), key=unit_key_func)], 1
     ):
-
-        # The indices of the X and Y coordinates in a list of point coords.
-        X = 0
-        Y = 1
-
         # Initialize data structures that store info for each side of a schematic symbol unit.
-        all_sides = ["left", "right", "top", "bottom"]
         bbox = {side: [(XO, YO), (XO, YO)] for side in all_sides}
-        box_pt = {side: [XO + PIN_LENGTH, YO + PIN_SPACING] for side in all_sides}
-        anchor_pt = {side: [XO + PIN_LENGTH, YO + PIN_SPACING] for side in all_sides}
+        box_pt = {side: [XO + pin_length, YO + PIN_SPACING] for side in all_sides}
+        anchor_pt = {side: [XO + pin_length, YO + PIN_SPACING] for side in all_sides}
         transform = {}
+        unitdata[unit_num] = ((bbox, box_pt, anchor_pt, transform))
 
         # Annotate the pins for each side of the symbol.
         for side_pins in list(unit.values()):
@@ -660,13 +582,13 @@ def draw_symbol(
 
         # Determine the actual bounding box for each side.
         for side, side_pins in list(unit.items()):
-            bbox[side] = pins_bbox(list(side_pins.items()))
+            bbox[side] = pins_bbox(list(side_pins.items()), pin_length)
 
         # Adjust the sizes of the bboxes to make the unit look more symmetrical.
         balance_bboxes(bbox)
 
         # Calculate the amount to move the left/right sides to move them under the top/bottom rows.
-        scrunch_offset = calc_scrunch(bbox) if scrunch else 0
+        scrunch_offset = calc_scrunch(bbox, pin_length) if scrunch else 0
 
         # Determine some important points for each side of pins.
         for side in unit:
@@ -685,7 +607,7 @@ def draw_symbol(
                 max(bbox[side][0][Y], bbox[side][1][Y]),
             ]
             box_pt[side] = [
-                min(bbox[side][0][X], bbox[side][1][X]) + PIN_LENGTH,
+                min(bbox[side][0][X], bbox[side][1][X]) + pin_length,
                 max(bbox[side][0][Y], bbox[side][1][Y]),
             ]
 
@@ -776,6 +698,92 @@ def draw_symbol(
                     * box_pt[side]
                 )
 
+    # Determine the field location
+    # If there are pins across the top of the symbol, right-justify the
+    # reference, part number, etc. so they don't run into the top pins.
+    # Otherwise, stick with left-justification.
+    text_justification = "L"
+    horiz_offset = int(box_pt["left"][X])
+    vert_offset  = int(box_pt["top"][Y]) + 150
+    if any("top" in unit.keys() for unit in pin_data.values()):
+        text_justification = "R"
+        horiz_offset -= 50
+
+    # Start the part definition with the header.
+    part_defn = START_DEF.format(
+        name=part_num,
+        ref=part_ref_prefix,
+        pin_name_offset=PIN_NAME_OFFSET,
+        show_pin_number=SHOW_PIN_NUMBER and "Y" or "N",
+        show_pin_name=SHOW_PIN_NAME and "Y" or "N",
+        num_units=len(pin_data),
+    )
+
+    # Create the field that stores the part reference.
+    part_defn += REF_FIELD.format(
+        ref_prefix=part_ref_prefix or "U",
+        x=XO + horiz_offset,
+        y=YO + REF_Y_OFFSET + vert_offset,
+        text_justification=text_justification,
+        font_size=REF_SIZE,
+    )
+
+    # Create the field that stores the part number.
+    part_defn += PARTNUM_FIELD.format(
+        num=part_num or "",
+        x=XO + horiz_offset,
+        y=YO + PART_NUM_Y_OFFSET + vert_offset,
+        text_justification=text_justification,
+        font_size=PART_NUM_SIZE,
+    )
+
+    # Create the field that stores the part footprint.
+    part_defn += FOOTPRINT_FIELD.format(
+        footprint=part_footprint or "",
+        x=XO + horiz_offset,
+        y=YO + PART_FOOTPRINT_Y_OFFSET + vert_offset,
+        text_justification=text_justification,
+        font_size=PART_FOOTPRINT_SIZE,
+    )
+
+    # Create the field that stores the datasheet link.
+    part_defn += DATASHEET_FIELD.format(
+        datasheet=part_datasheet or "",
+        x=XO + horiz_offset,
+        y=YO + PART_DATASHEET_Y_OFFSET + vert_offset,
+        text_justification=text_justification,
+        font_size=PART_DATASHEET_SIZE,
+    )
+
+    # Create the field that stores the manufacturer part number.
+    if part_manf_num:
+        part_defn += MPN_FIELD.format(
+            manf_num=part_manf_num,
+            x=XO + horiz_offset,
+            y=YO + PART_MPN_Y_OFFSET + vert_offset,
+            text_justification=text_justification,
+            font_size=PART_MPN_SIZE,
+        )
+
+    # Create the field that stores the datasheet link.
+    if part_desc:
+        part_defn += DESC_FIELD.format(
+            desc=part_desc,
+            x=XO + horiz_offset,
+            y=YO + PART_DESC_Y_OFFSET + vert_offset,
+            text_justification=text_justification,
+            font_size=PART_DESC_SIZE,
+        )
+
+    # Start the section of the part definition that holds the part's units.
+    part_defn += START_DRAW
+
+    # Now create the units that make up the part.
+    for unit_num, unit in enumerate(
+        [p[1] for p in sorted(pin_data.items(), key=unit_key_func)], 1
+    ):
+        bbox, box_pt, anchor_pt, transform = unitdata[unit_num]
+
         # Draw the transformed pins for each side of the symbol.
         for side in all_sides:
             side_pins = unit[side]
@@ -799,6 +807,7 @@ def draw_symbol(
                 side,
                 push,
                 fuzzy_match,
+                pin_length,
             )
 
         # Create the box around the unit's pins.
@@ -822,6 +831,20 @@ def draw_symbol(
     return part_defn
 
 
+def calculate_pin_length(pin_data, fuzzy_match):
+    pin_length = 0
+    for unit in pin_data.values():
+        for side in unit.values():
+            for pins in side.values():
+                for p in pins:
+                    style = find_closest_match(p.style, PIN_STYLES, fuzzy_match)
+                    pin_num, _ = get_pin_num_and_spacer(p)
+                    pin_length = max(pin_length,
+                            PIN_LENGTH_PER_CHAR * len(pin_num) +
+                            PIN_LENGTH_STYLE_MOD * (style in PIN_LENGTH_STYLES))
+    return max(PIN_LENGTH_BASE + pin_length, MIN_PIN_LENGTH)
+
+
 def can_bundle(pin, fuzzy_match):
     """Return true if the pin type can be bundled.
     Currently support bundling power input, power output, and NC pins.
